From 37b7ba489f52ac7588da4b230c053194284ecc0d Mon Sep 17 00:00:00 2001
From: Min Xu <min.xu[A.T]min-info[D.O.T]net>
Date: Thu, 5 Sep 2013 16:04:25 -1000
Subject: [PATCH 2/6] Ensures the current req_ctx is transmitted to host when
 any of the following occurs:

1) VCC_PHONE off
2) nRST
3) RST

When those event occurs, IF there are data in existing req_ctx, they
are to be transmitted before any state change in iso uart struct.  So that
an ATR flag in the req_ctx (received on Host software via USB) will always
indicate that ATR starts at position 0 (the host software already expects
this)
---
 firmware/src/simtrace/iso7816_uart.c |    3 +++
 firmware/src/simtrace/iso7816_uart.h |    4 ++++
 firmware/src/simtrace/sim_switch.c   |    6 ++++--
 3 files changed, 11 insertions(+), 2 deletions(-)

diff --git a/firmware/src/simtrace/iso7816_uart.c b/firmware/src/simtrace/iso7816_uart.c
index e4c0a82..9f7c143 100644
--- a/firmware/src/simtrace/iso7816_uart.c
+++ b/firmware/src/simtrace/iso7816_uart.c
@@ -617,10 +617,13 @@ static __ramfunc void usart_irq(void)
 /* handler for the RST input pin state change */
 static void reset_pin_irq(u_int32_t pio)
 {
+	// When ever reset occured, we make sure to flush out any populated req_ctx
 	if (!AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, pio)) {
+		iso7816_wtime_expired();
 		DEBUGPCR("nRST");
 		set_state(&isoh, ISO7816_S_RESET);
 	} else {
+		iso7816_wtime_expired();
 		DEBUGPCR("RST");
 		set_state(&isoh, ISO7816_S_WAIT_ATR);
 		isoh.stats.rst++;
diff --git a/firmware/src/simtrace/iso7816_uart.h b/firmware/src/simtrace/iso7816_uart.h
index beb0b23..68615a3 100644
--- a/firmware/src/simtrace/iso7816_uart.h
+++ b/firmware/src/simtrace/iso7816_uart.h
@@ -1,3 +1,5 @@
+#ifndef SIMTRACE_ISO7816_UART_H
+#define SIMTRACE_ISO7816_UART_H
 
 struct simtrace_stats *iso_uart_stats_get(void);
 void iso_uart_stats_dump(void);
@@ -6,3 +8,5 @@ void iso_uart_rst(unsigned int state);
 void iso_uart_rx_mode(void);
 void iso_uart_clk_master(unsigned int master);
 void iso_uart_init(void);
+
+#endif
diff --git a/firmware/src/simtrace/sim_switch.c b/firmware/src/simtrace/sim_switch.c
index faf48f0..a374e7d 100644
--- a/firmware/src/simtrace/sim_switch.c
+++ b/firmware/src/simtrace/sim_switch.c
@@ -59,10 +59,12 @@ static void sw_sim_irq(u_int32_t pio)
 
 static void vcc_phone_irq(u_int32_t pio)
 {
-	if (!AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, SIMTRACE_PIO_VCC_PHONE))
+	if (!AT91F_PIO_IsInputSet(AT91C_BASE_PIOA, SIMTRACE_PIO_VCC_PHONE)) {
 		DEBUGPCR("VCC_PHONE off");
-	else
+		iso7816_wtime_expired();
+	} else {
 		DEBUGPCR("VCC_PHONE on");
+	}
 }
 
 void sim_switch_init(void)
-- 
1.7.9.5

